; =====================================================================
; Dekodierung PPM-Summensignal -> Servoimpulse mit ATTiny2313
; =====================================================================
;
; =====================================================================
; Erstellt von Jrg Reisener am:			28.01.2009
; letzte Bearbeitung:					28.01.2009
; =====================================================================
;
; Einstellung der Konfigurationsbits (Fuse-Bits) des Tiny2313:
; (interner 8MHz-Takt ohne Vorteiler)
;
; CKDIV8			 = 1
; CKOUT				 = 1
; SUT1, SUT0			 = 10
; CKSEL3, CKSEL2, CKSEL1, CKSEL0 = 0100
; alle anderen			 = 1 (unchecked)
; =====================================================================
;
; ----------------------
; Belegung der I/O-Ports
; ----------------------
;
; PortB0: Ausgang Servokanal 1
; PortB1: Ausgang Servokanal 2
; PortB2: Ausgang Servokanal 3
; PortB3: Ausgang Servokanal 4
; PortB4: Ausgang Servokanal 5
; PortB5: (MOSI) Ausgang Servokanal 6
; PortB6: (MISO) Ausgang Servokanal 7
; PortB7: (SCK) Ausgang debug (Startimpulslnge)
;
; PortD0: nicht genutzt
; PortD1: nicht genutzt
; PortD2: Eingang PPM-Signal (INT0)
; PortD3: nicht genutzt
; PortD4: nicht genutzt
; PortD5: nicht genutzt
; PortD6: nicht genutzt
;
; ------------------
; Belegung der Timer
; ------------------
;
; Timer0: Messung Startimpuls
; Timer1: nicht genutzt
;
; -----------------------------------
; Konstanten und Registerdefinitionen
; -----------------------------------
;
.nolist
.include "tn2313def.inc"
.list
;
.equ	xtal=	8000000	;Controller Taktfrequenz 8 MHz, intern
;
.def	insreg=	r1	;Zwischenspeicher fr SREG whrend Interrupt
.def	kanal=	r20	;Kanalzhler
.def	itemp=	r25	;Zwischenspeicher whrend Interruptbearbeitung
; --------------
; SRAM Variablen
; --------------
;
.dseg
.org	0x60
;
; ---------------
; Programmbereich
; ---------------
;
.cseg
.org	0
;
; Interrupt-Vektoren
;
reset:	rjmp	start			;Programmstart
	rjmp	exint0			;ext. Interrupt PPM-Signal
	reti
	reti
	reti
	reti
	reti
	reti
	reti
	reti
	reti
	reti
	reti
	rjmp	t0comp			;Timer0 Compare A Interrupt
;
.db	"                    "
.db	"ppm to servoimpuls  "
.db	"01/2009             "
.db	"Joerg Reisener      "
.db	"                    "
;
; Initialisierung SRAM, I/O-Ports, Hardware
;
start:	ldi	zl,0x60
	ldi	zh,0			;Anfangsadresse SRAM
	clr	r16
sramcl:	st	z+,r16			;Speicherzelle lschen
	cpi	zl,ramend+1		;letzte Speicherzelle erreicht?
	brne	sramcl			;nein -> Schleife
	ldi	r16,ramend
	out	spl,r16			;Stackpointer setzen
	clr	r16
;
	ldi	r16,0b11111111		;PortB, B7-B0 Aus
	out	ddrb,r16
	ldi	r16,0b00000000		;PortB, B7-B0 low
	out	portb,r16
	ldi	r16,0b00000000		;PortD, D6-D0 Ein
	out	ddrd,r16
	ldi	r16,0b01111111		;PortD, D6-D0 Pullup
	out	portd,r16
;
	ldi	r16,(1<<wgm01)		;Timer0 CTC-Mode
	out	tccr0a,r16
	ldi	r16,(1<<cs02)		;Vorteiler 256
	out	tccr0b,r16
	ldi	r16,78			;Impulspause mind. 2,5ms
	out	ocr0a,r16		;Timer Compare laden
	ldi	r16,(1<<ocie0a)		;erlaube Compare Match Int.
	out	timsk,r16
;
	ldi	r16,(1<<isc01)		;INT0 fallende Flanke
	out	mcucr,r16
	ldi	r16,(1<<int0)		;erlaube Interrupt
	out	gimsk,r16
;
	rcall	w25ms
	rcall	w25ms
	rcall	w25ms
	ldi	kanal,1			;Kanalzhler rcksetzen
	sei				;Interrupt freigeben
;
; Hauptprogramm
;
main:
	rjmp	main			;Endlosschleife
;
; ----------------------
; Unterprogramm-Routinen
; ----------------------
;
; Warteschleife 25ms
;
w25ms:	push	r16
	push	r17
	clr	r16
	clr	r17
wloop:	dec	r16			;innere Schleife
	brne	wloop			;luft 96ys
	dec	r17
	brne	wloop			;uere Schleife
	pop	r17
	pop	r16
	ret
;
; ------------------
; Interrupt-Routinen
; ------------------
;
; Externer Interrupt INT0
;
exint0:	in	insreg,sreg		;SREG sichern
	out	portb,kanal		;neuen Kanalimpuls ausgeben
	lsl	kanal			;nchsten Kanal whlen
	clr	itemp
	out	tcnt0,itemp		;Timer0 zurcksetzen
ex0090:	out	sreg,insreg		;SREG wiederherstellen
	reti
;
; Timer 0 Compare Match A Interrupt
;
t0comp:	in	insreg,sreg		;SREG sichern
	ldi	kanal,1			;Kanalzhler rcksetzen
	clr	itemp
	out	portb,itemp		;Ende debug-Impuls
	out	sreg,insreg		;SREG wiederherstellen
	reti
;
